
package testClientSoap;

import java.util.*;
import java.net.*; 
import java.rmi.*; 
import javax.xml.namespace.*; 
import javax.xml.rpc.*; 
import javax.wsdl.OperationType;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.client.Call; 
import org.apache.axis.client.Service; 
import org.apache.axis.utils.*;

/*
    test class to access a web service exposed by Ofbiz
*/ 
public class Hello3ClientSoap { 
    
    public static void main(String[] args) { 
        
        String message = ""; 
        Map output;
        ArrayList outputList;
        String helloPersonIdOut,firstName,lastName,endpoint,inputParam;
    
    try { 
        // note that the port is 18080 because I am using the TCP monitor
        // to trace the SOAP requests and responses
        // Ofbiz is still responding on the port 8080 
        endpoint = "http://127.0.0.1:18080/webtools/control/SOAPService/";
        // input parameter to pass
        inputParam = "10000";
        
        // create the standard JAX-RPC Call object
        Call call = (Call) new Service().createCall(); 
        // set the endpoint
        call.setTargetEndpointAddress(new URL(endpoint)); 
        // set the WS name and method to invoke
        call.setOperationName(new QName("searchHelloPerson", "searchHelloPerson")); 
        // define the name of the input parameter to be passed into the
        // SOAP request. This is necessary otherwise the the service implemented
        // in Ofbiz will fail during the validation phase 
        call.addParameter("helloPersonId",
                          org.apache.axis.Constants.XSD_STRING,
                          javax.xml.rpc.ParameterMode.IN);
        call.setReturnType(org.apache.axis.Constants.XSD_STRING);

        // execute the remote method exposed by the WS passing 
        // the input parameter
        // responseWS will contain the first returned parameter (helloPersonIdOut)
        // it is a strange behaviour but axis 1.4 works like that, 
        // look at /src/org/apache/axis/client/Call.java:2528
        Object responseWS = call.invoke(new Object[]{inputParam}); 
        System.out.println( "Receiving response: " +  (String) responseWS); 

      
        // retrieve the other parameters via getOutputParams() method
        // this will identify the parameters through their names
        output = call.getOutputParams();
        // get the output parameter helloPersonIdOut
        // just for fun, the first parameter returned is not here, but in the response
        try {
            helloPersonIdOut = (String) output.get(new QName("", "helloPersonIdOut"));
        } catch (Exception _exception) {
            helloPersonIdOut = (String) org.apache.axis.utils.JavaUtils.convert(output.get(new QName("", "helloPersonIdOut")), java.lang.String.class);
        }
        // get the output parameter firstName
        try {
            firstName = (String) output.get(new QName("", "firstName"));
        } catch (Exception _exception) {
            firstName = (String) org.apache.axis.utils.JavaUtils.convert(output.get(new QName("", "firstName")), java.lang.String.class);
        }
        // get the output parameter lastName
        try {
            lastName = (String) output.get(new QName("", "lastName"));
        } catch (Exception _exception) {
            lastName = (String) org.apache.axis.utils.JavaUtils.convert(output.get(new QName("", "lastName")), java.lang.String.class);
        }
        System.out.println( "Reading parameter by name: helloPersonIdOut = " +  helloPersonIdOut); 
        System.out.println( "Reading parameter by name: firstName        = " +  firstName); 
        System.out.println( "Reading parameter by name: lastName         = " +  lastName); 

      
        // just to try a different way to get output parameters
        // retrieve the other parameters via getOutputValues() method
        // this will identify the parameters through their position
        outputList = (ArrayList) call.getOutputValues();
        // even here the first output parameter is missing
        for (int i=0;i<outputList.size();i++){
            System.out.println("Reading parameter by position: [" + i + "]  = " + outputList.get(i));
        }

    } catch (MalformedURLException ex) { 
        message = "error: wrong url"; 
    } catch (ServiceException ex) { 
        message = "error: failed to create the call"; 
    } catch (RemoteException ex) { 
        message = "error: failed to invoke WS";
    } finally { 
      System.out.println(""); 
      System.out.println(message); 
    } 
  } 
}     